
import os
import datetime
import urllib

from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.action_chains import ActionChains

from promium.waits import wait_for_page_loaded


def scroll_to_bottom(driver):
    """Scrolls down page"""
    driver.execute_script('jQuery("img.img-ondemand").trigger("appear");')
    wait_for_page_loaded(driver)
    driver.execute_script(
        """
        var f = function(old_height) {
            var height = $$(document).height();
            if (height == old_height) return;
            $$('html, body').animate({scrollTop:height}, 'slow', null,
            setTimeout(function() {f(height)}, 1000)
            );
        }
        f();
        """
    )


def scroll_to_top(driver):
    wait_for_page_loaded(driver)
    driver.execute_script(
        """jQuery('html, body').animate({scrollTop: 0 }, 'slow', null);"""
    )


def scroll_to_bottom_in_block(driver, element_class):
    script = """
        var elem = '.'.concat(arguments[0]);
        $(elem).animate({scrollTop: $(elem).prop('scrollHeight')}, 1000);
    """
    driver.execute_script(script, element_class)


def scroll_to_element(driver, element, base_element=None):
    """
    use base_element if you need for example scroll into popup,
    base_element must be a popup locator.
    """
    if base_element is None:
        base_element = 'html, body'
    script = """
        var elem = arguments[0];
        var base = arguments[1];
        var relativeOffset = (
            jQuery(elem).offset().top - jQuery(base).offset().top
        );
        jQuery(base).animate({
            scrollTop: relativeOffset
            }, 'slow', null
        );
             """
    wait_for_page_loaded(driver)
    driver.execute_script(script, element, base_element)


def scroll_with_offset(driver, element, with_offset=0):
    script = """
        var elem = arguments[0];
        jQuery('html, body').animate({
            scrollTop: jQuery(elem).offset().top + arguments[1]
            }, 'fast', null
        );
        """
    driver.execute_script(script, element, with_offset)


# TODO need implemented
def switch_to_window(driver, window_handle):
    """Switches to window"""
    driver.switch_to_window(window_handle)


# TODO need implemented save to storage
def get_screenshot(driver):
    """Gets screenshot"""
    now = datetime.datetime.now().strftime('%d_%H_%M_%S_%f')
    path = os.path.abspath(os.curdir)
    if ':' in path:
        screenshots_folder = path + '\screenshots\\'
    else:
        screenshots_folder = path + '/screenshots/'
    screenshot_name = "{name}_{time}.png".format(name='screenshot', time=now)
    url = screenshots_folder + screenshot_name
    driver.save_screenshot(url)
    return url


def download_and_check_file(download_url, file_path):
    try:
        urllib.urlretrieve(download_url, file_path)
        if os.path.isfile(file_path) and os.path.exists(file_path):
            return True
        else:
            return False
    finally:
        if os.path.exists(file_path):
            os.remove(file_path)


def control_plus_key(driver, key):
    """Imitations press CONTROL key + any key"""
    (
        ActionChains(driver)
        .key_down(Keys.CONTROL)
        .send_keys(key)
        .key_up(Keys.CONTROL)
        .perform()
    )


def set_local_storage(driver, key, value):
    """Sets value in browsers local storage"""
    driver.execute_script(
        "localStorage.setItem('%s', '%s')" % (key, value)
    )


def delete_cookie_item(driver, name):
    driver.delete_cookie(name)
    driver.refresh()


def delete_element_from_dom(driver, element):
    driver.execute_script(f"""
        var element = document.querySelector("{element}");
        if (element)
        element.parentNode.removeChild(element);
    """)


def scroll_into_end(driver, timeout=1):
    """Scroll block into end page"""
    driver.execute_script(
        'window.scrollTo(0, document.body.scrollHeight);'
    )
    datetime.time.sleep(timeout)




import pytest

from promium.device_config import ANDROID_360_640
from promium.test_case import WebDriverTestCase
from tests.pages.catalog_gallery_page import CatalogGalleryPage


@pytest.mark.se
class TestCatalogGallery(WebDriverTestCase):

    def test_view_products_gallery_blocks(self):
        catalog_gallery_page = CatalogGalleryPage(self.driver)
        catalog_gallery_page.open(query=u'Komplekty-nizhnego-belya')
        products_block = catalog_gallery_page.products_block
        products_block.wait_to_display()
        products = products_block.product_blocks
        product = products.first_item.wait_to_display()
        product.hover_over()
        show_all_phones_link = product.show_all_phones_link
        show_all_phones_link.wait_to_display()
        catalog_gallery_page.show_more_button.wait_to_display()

    @pytest.mark.device(ANDROID_360_640)
    def test_mobile_view_products_gallery_blocks(self):
        catalog_gallery_page = CatalogGalleryPage(self.driver)
        catalog_gallery_page.open(query=u'Komplekty-nizhnego-belya')
        products_block = catalog_gallery_page.products_block.wait_to_display()
        products_block.product_blocks.first_item.wait_to_display()



import sys
import requests


GITHUB_URL = 'https://api.github.com/repos'
GECKODRIVER = {
    'url': '{}/mozilla/geckodriver/releases/latest'.format(GITHUB_URL),
    'key': 'tag_name'
}
OPERADRIVER = {
    'url': '{}/operasoftware/operachromiumdriver/releases/latest'.format(
        GITHUB_URL
    ),
    'key': 'name'
}


def get_driver_last_version(driver_name):
    if driver_name == 'geckodriver':
        driver = GECKODRIVER
    elif driver_name == 'operadriver':
        driver = OPERADRIVER
    else:
        raise Exception('Driver name must be "geckodriver" or "operadriver"')
    response = requests.get(url=driver['url'], verify=False, timeout=10).json()
    version = response[driver['key']]
    return version


if __name__ == "__main__":
    print(get_driver_last_version(sys.argv[1]))



import requests
from bs4 import BeautifulSoup


def get_opera_last_version():
    response = requests.get(
        url='https://ftp.opera.com/pub/opera/desktop/',
        verify=False,
        timeout=10
    )
    soup = BeautifulSoup(response.text, 'html.parser')
    links = soup.findAll('a')
    versions = [
        link.get('href').replace('/', '')
        for link in links
        if link.get('href') not in ['../', 'desktop/']
    ]
    last_version = versions[-1]
    return last_version


if __name__ == "__main__":
    print(get_opera_last_version())



# -*- coding: utf-8 -*-

import pytest
from bs4 import BeautifulSoup

from promium.test_case import RequestTestCase
from tests.urls import collect_url

from .pages.catalog_main_page import CatalogMainPage


BLOCKS = [
    'footer_customers',
    'footer_vendors',
    'footer_about_as',
    'footer_partners',
    'footer_countries',
    'top-searches',
    'footer_social_links'
]


@pytest.mark.request
class TestMainPage(RequestTestCase):

    @pytest.mark.parametrize('block', BLOCKS)
    def test_main_page_links(self, block):
        page_text = self.get_response(CatalogMainPage.url).text
        soup = BeautifulSoup(page_text, 'html.parser')
        needful_block = soup.find(attrs={'data-qaid': block})
        links = needful_block.findAll('a')
        for link in links:
            href = link.get('href')
            url = collect_url(href.strip('/')) if '://' not in href else href
            if "https://vk.com" not in url:
                response = self.get_response(url)
                self.soft_assert_equals(
                    response.status_code,
                    200,
                    u'Не доступна страница по url={} на главной'.format(url)
                )



# -*- coding: utf-8 -*-

from selenium.webdriver.common.by import By

from promium import Page, Block, Element, Link, InputField
from tests.urls import collect_url


class CheckBoxBlock(Block):
    pass


class WithCheckBoxesBlock(Block):

    in_category_block = CheckBoxBlock(
        By.CSS_SELECTOR,
        '[data-qaid="category_line"]'
    )
    in_tag_block = CheckBoxBlock(
        By.CSS_SELECTOR,
        '[data-qaid="tag_line"]'
    )
    in_company_block = CheckBoxBlock(
        By.CSS_SELECTOR,
        '[data-qaid="company_line"]'
    )


class SearchTermBlock(Block):

    hits_term = Element(
        By.CSS_SELECTOR,
        '[data-qaid="term_hits_text"]'
    )
    main_term = Element(
        By.CSS_SELECTOR,
        '[data-qaid="term_main_text"]'
    )


class SuggestItemBlock(Block):

    category_term_link = Element(
        By.CSS_SELECTOR,
        '[data-qaid="category_term_link"]'
    )
    search_term_link = SearchTermBlock(
        By.CSS_SELECTOR,
        '[data-qaid="search_term_link"]'
    )


class SuggestItemsBlock(Block):

    suggest_item_block = SuggestItemBlock.as_list(
        By.CSS_SELECTOR,
        '[data-qaid="suggest_item"]'
    )


class SuggesterBlock(Block):

    checkbox_block = WithCheckBoxesBlock(
        By.CSS_SELECTOR,
        '[data-qaid="group_with_checkbox"]'
    )
    all_results_block = Block(
        By.CSS_SELECTOR,
        '[data-qaid="all_results_list"]'
    )
    suggest_items_block = SuggestItemsBlock.as_list(
        By.CSS_SELECTOR,
        '[data-qaid="autocomplete_list"]'
    )


class SearchBlock(Block):

    search_input = InputField(
        By.CSS_SELECTOR,
        '[data-qaid="search_input"]'
    )
    search_button = Element(
        By.CSS_SELECTOR,
        '[data-qaid="search_button"]'
    )
    suggester_block = SuggesterBlock(
        By.CSS_SELECTOR,
        '[data-qaid="autocomplete_block"]'
    )


class Catalog404WrapperBlock(Block):

    search_block = SearchBlock(
        By.CSS_SELECTOR,
        '[data-qaid="search_input_block"]'
    )

    logo_link = Link(
        By.CSS_SELECTOR,
        '[data-qaid="404_logo_href"]'
    )

    main_404_title = Element(
        By.CSS_SELECTOR,
        '[data-qaid="404_main_title"]'
    )

    suggest_404_title = Element(
        By.CSS_SELECTOR,
        '[data-qaid="404_suggest_title"]'
    )

    suggest_404_title_link = Link(
        By.CSS_SELECTOR,
        '[data-qaid="404_suggest_title_link"]'
    )


class Catalog404Page(Page):

    url = collect_url('xxx')

    wrapper_block = Catalog404WrapperBlock(
        By.CSS_SELECTOR,
        '[data-qaid="404_wrapper_block"]'
    )



# -*- coding: utf-8 -*-

import os
import datetime
import logging
import pytest

from selenium.common.exceptions import (
    WebDriverException,
    ElementNotVisibleException,
    InvalidElementStateException,
    InvalidSelectorException,
    NoSuchElementException,
    StaleElementReferenceException
)

from promium.logger import Logger
from promium.core.exceptions import BrowserConsoleException


log = logging.getLogger(__name__)


def pytest_sessionstart(session):
    if hasattr(session.config, 'cache'):
        cache = session.config.cache
        cache_path = u"cache/{}".format(session.config.lastfailed_file)
        print(u"Lastfailed:", cache.get(cache_path, set()))
    session.run_duration = datetime.datetime.now()
    print(u"\nPytest session start %s\n" % session.run_duration)


@pytest.mark.trylast
def pytest_sessionfinish(session):
    finish = datetime.datetime.now()
    duration = datetime.timedelta(
        seconds=(finish - session.run_duration).seconds
    )
    print(u"\n\nPytest session finish %s (duration=%s)\n" % (finish, duration))


@pytest.hookimpl
def pytest_addoption(parser):
    parser.addoption(
        "--capturelog",
        dest="logger",
        default=None,
        action="store_true",
        help="Show log messages for each failed test"
    )
    parser.addoption(
        "--fail-debug-info",
        action="store_true",
        help="Show screenshot and test case urls for each failed test"
    )
    parser.addoption(
        "--duration-time",
        action="store_true",
        help="Show the very slow test"
    )
    parser.addoption(
        "--highlight",
        action="store_true",
        help="Highlighting elements"
    )
    parser.addoption(
        "--headless",
        action="store_true",
        help="Enable headless mode for Chrome browser only"
    )
    parser.addoption(
        "--chrome",
        action="store_true",
        help="Use chrome browser"
    )
    parser.addoption(
        "--firefox",
        action="store_true",
        help="Use firefox browser"
    )
    parser.addoption(
        "--opera",
        action="store_true",
        help="Use opera browser"
    )
    parser.addoption(
        "--ie",
        action="store_true",
        help="Use ie browser"
    )
    parser.addoption(
        "--edge",
        action="store_true",
        help="Use edge browser"
    )
    parser.addoption(
        "--grid",
        help=(
            "Use selenium grid host for example: "
            "'http://192.168.0.1:4444/wd/hub' "
            "or  with base auth "
            "'http://test:test-password@192.168.0.1:4444/wd/hub'"
        )
    )
    parser.addoption(
        "--repeat",
        action="store",
        default=1,
        type="int",
        metavar='repeat',
        help='Number of times to repeat each test. Mostly for debug purposes'
    )
    parser.addoption(
        "--check-console",
        action="store_true",
        help="Check browser console js and other errors"
    )


def pytest_generate_tests(metafunc):
    if metafunc.config.option.repeat > 1:
        metafunc.fixturenames.append('repeat')
        metafunc.parametrize('repeat', range(metafunc.config.option.repeat))


@pytest.fixture(autouse=True)
def logger(request):
    return logging.getLogger()


@pytest.mark.trylast
def pytest_runtest_call(item):
    if hasattr(item.config, "assertion_errors"):
        if item.config.assertion_errors:
            msg = u"\n{assertion_errors_list}\n".format(
                assertion_errors_list="\n".join(item.config.assertion_errors)
            )
            raise AssertionError(msg)
    if item.config.getoption("--check-console"):
        if hasattr(item.config, "check_console_errors"):
            browser_console_errors = item.config.check_console_errors()
            if browser_console_errors:
                msg = (
                    u"Browser console js errors found:"
                    u"\n{console_errors}\n".format(
                        console_errors=u"\n".join(
                            browser_console_errors
                        )
                    )
                )
                raise BrowserConsoleException(msg)


@pytest.mark.tryfirst
@pytest.mark.hookwrapper
def pytest_runtest_makereport(item, call):
    """pytest failed test report generator"""
    html_plugin = item.config.pluginmanager.getplugin('html')
    outcome = yield
    report = outcome.get_result()
    when = report.when
    excinfo = call.excinfo
    extra = getattr(report, 'extra', [])

    # remove webdriver java traceback
    if excinfo:
        if excinfo.type in [
            ElementNotVisibleException,
            InvalidElementStateException,
            InvalidSelectorException,
            NoSuchElementException,
            StaleElementReferenceException
        ] and report.longrepr:
            report.longrepr.reprtraceback.reprentries[-1].lines = \
                report.longrepr.reprtraceback.reprentries[-1].lines[:2]

    if when == "call" and not report.passed:
        if item.config.getoption("--fail-debug-info"):
            fail_info = u"Fail info not found."
            try:
                if hasattr(item.config, "get_fail_debug"):
                    fail_debug = item.config.get_fail_debug()
                    is_vagga = False
                    for k in (dict(os.environ)).keys():
                        if 'VAGGA' in k:
                            is_vagga = True
                            break
                    if fail_debug[0] == 'webdriver':
                        (
                            url,
                            screenshot,
                            test_case_url,
                            pytest_failed_test_command,
                            vagga_failed_test_command
                        ) = fail_debug[1:]
                        test_case = (
                            u"\nTEST CASE: {}".format(test_case_url)
                            if test_case_url else u''
                        )
                        run_test_command = (
                            u"\nVAGGA COMMAND: {}".format(
                                vagga_failed_test_command
                            ) if is_vagga
                            else u"\nPYTEST COMMAND: {}".format(
                                pytest_failed_test_command
                            )
                        )
                        fail_info = (
                            (
                                u"\nURL: {url}"
                                u"{test_case_url}"
                                u"\nSCREENSHOT: {screenshot}"
                                u"{run_test_command}"
                                u"\n"
                            ).format(
                                url=url,
                                screenshot=screenshot,
                                test_case_url=test_case,
                                run_test_command=run_test_command,
                            )
                        )
                        extra.append(html_plugin.extras.url(
                            screenshot, name=u"SCREENSHOT"
                        ))
                        extra.append(
                            html_plugin.extras.html(
                                u"<img src=\"{img}\" "
                                u"height=\"230\" width=\"auto\" align=\""
                                u"right\" bottom=\"0px\">Screenshot</img>"
                                .format(
                                    img=screenshot
                                )
                            )
                        )
                    elif fail_debug[0] == 'request':
                        (
                            url,
                            status_code,
                            test_case_url,
                            pytest_failed_test_command,
                            vagga_failed_test_command
                        ) = fail_debug[1:]
                        test_case = (
                            u"\nTEST CASE: {}".format(test_case_url)
                            if test_case_url else u''
                        )
                        run_test_command = (
                            u"\nVAGGA COMMAND: {}".format(
                                vagga_failed_test_command
                            ) if is_vagga
                            else u"\nPYTEST COMMAND: {}".format(
                                pytest_failed_test_command
                            )
                        )
                        fail_info = (
                            u"\nURL: {url} (status code: {status_code})"
                            u"{test_case_url}"
                            u"{run_test_command}"
                            u"\n"
                        ).format(
                            url=url,
                            status_code=status_code,
                            test_case_url=test_case,
                            run_test_command=run_test_command,
                        )
                    extra.append(html_plugin.extras.url(
                        url, name=u"URL"
                    ))
                    extra.append(html_plugin.extras.url(
                        test_case_url, name=u"TEST CASE"
                    ))
                    report.extra = extra

            except WebDriverException as e:
                fail_info = (
                    u"\nWebdriver instance must have crushed: {msg}".format(
                        msg=e.msg
                    )
                )
            finally:
                longrepr = getattr(report, 'longrepr', None)
                if hasattr(longrepr, 'addsection'):
                    longrepr.sections.insert(
                        0, (u"Captured stdout %s" % when, fail_info, u"-")
                    )

        if item.config.getoption("--duration-time"):
            duration = call.stop - call.start
            if duration > 120:
                report.sections.append((
                    u"Captured stdout %s" % when,
                    (u"\n\n!!!!! The very slow test. "
                     u"Duration time is %s !!!!!\n\n")
                    % datetime.datetime.fromtimestamp(duration).strftime(
                        u"%M min %S sec"
                    )
                ))


def pytest_configure(config):
    os.environ['PYTHONWARNINGS'] = (
        u'ignore:An HTTPS request has been made, '
        u'ignore:A true SSLContext object is not available, '
        u'ignore:Unverified HTTPS request is being made'
    )
    grid_host = config.getoption("--grid", '')
    if grid_host:
        os.environ['GRID_HOST'] = grid_host

    driver = 'chrome'
    if config.getoption("--firefox"):
        driver = 'firefox'
    elif config.getoption("--opera"):
        driver = 'opera'
    elif config.getoption("--ie"):
        driver = 'ie'
    elif config.getoption("--edge"):
        driver = 'edge'
    os.environ['SE_DRIVER'] = driver

    if config.getoption('--headless'):
        os.environ['HEADLESS'] = 'Enabled'
    if config.getoption("--highlight"):
        os.environ['HIGHLIGHT'] = 'Enabled'
    if config.getoption("--capturelog"):
        config.pluginmanager.register(Logger(), "logger")
    if config.getoption("--check-console"):
        os.environ['CHECK_CONSOLE'] = 'Enabled'
    config.lastfailed_file = "lastfailed"



# -*- coding: utf-8 -*-

WIKI_HOST = ''
PROJECT_HOST = 'prom.ua'
PROJECT_PORT = ''
GRID_HOST_PORT = '0.0.0.0:13400'
DB = ''
CHECK_TEST_CASE = False

OPERA_BINARY_LOCATION = '/usr/bin/opera'

# for windows 7
# OPERA_BINARY_LOCATION = r'C:\Program Files\Opera\launcher.exe'

# for windows 10
# import os
# OPERA_BINARY_LOCATION = (
#     r'C:\Users\{}\AppData\Local\Programs\Opera\launcher.exe'.format(
#         os.environ.get('USERNAME')
#     )
# )



# -*- coding: utf-8 -*-

import pytest

from promium.test_case import WebDriverTestCase
from tests.pages.catalog_main_page import CatalogMainPage


@pytest.mark.se
class TestCatalogHeader(WebDriverTestCase):

    def test_check_catalog_header_elements(self):
        main_page = CatalogMainPage(self.driver)
        main_page.open()
        header_block = main_page.header_block
        header_block.registration_link.hover_over()
        header_block.header_reg_popup.wait_to_display()
        header_block.logo_link.hover_over()
        self.soft_assert_equals(
            header_block.logo_link.get_status_code,
            200
        )
        self.soft_assert_equals(
            header_block.search_block.search_input.placeholder,
            u'Я ищу...'
        )
        self.soft_assert_equals(
            header_block.favourite_link.get_status_code,
            200
        )
        header_block.shopping_cart_link.click()
        main_page.empty_order_popup.wait_to_display()
        self.soft_assert_equals(
            main_page.header_block.authorization_link.get_status_code,
            200
        )



# -*- coding: utf-8 -*-

import logging
import os
import py
import pytest
import re
from functools import wraps

from selenium.common.exceptions import TimeoutException


log = logging.getLogger(__name__)


MAX_SYMBOLS = 255
TABS_FORMAT = u" " * 20

MINOR_ERRORS = []
MAJOR_ERRORS = []
SKIPPED_ERRORS = MINOR_ERRORS + MAJOR_ERRORS


def repr_console_errors(console_errors, tabs=TABS_FORMAT):
    return u"\n{tabs_format}".format(tabs_format=tabs).join(
        u">>> [CONSOLE ERROR] %s" % error for error in set(console_errors)
    )


def repr_args(*args, **kwargs):
    return u"{args}{mark}{kwargs}".format(
        args=u", ".join(list(map(lambda x: x, args))) if args else u"",
        kwargs=u", ".join(
            u"%s=%s" % (k, v) for k, v in kwargs.iteritems()
        ) if kwargs else u"",
        mark=u", " if args and kwargs else u""
    )


def is_error_in_skipped_list(e, err):
    if e not in MINOR_ERRORS and e in err:
        log.info(u">>> [SKIPPED ERROR] %s" % err)
    if e in err:
        return True
    return False


def find_console_browser_errors(driver):
    return list(map(
        lambda x: x["message"],
        list(filter(
            lambda x: x["level"] == "SEVERE" and (
                x if not list(filter(
                    lambda e: True if is_error_in_skipped_list(
                        e, x["message"]
                    ) else False,
                    SKIPPED_ERRORS
                )) else None
            ), driver.get_log("browser")
        ))
    ))


def logger_for_element_methods(fn):

    @wraps(fn)
    def wrapper(self, *args, **kwargs):
        try:
            res = fn(self, *args, **kwargs)
        except TimeoutException as e:
            console_errors = find_console_browser_errors(self.driver)
            is_check = os.environ.get('CHECK_CONSOLE')
            if console_errors:
                try:
                    log.warning(
                        u"Browser console js error found:\n"
                        u"{tabs_format}{console_errors}\n"
                        u"{tabs_format}Url: {url}\n"
                        u"{tabs_format}Action: {class_name}"
                        u"({by}={locator})"
                        u".{method}({args})".format(
                            tabs_format=TABS_FORMAT,
                            class_name=self.__class__.__name__,
                            by=self.by,
                            locator=self.locator,
                            method=fn.__name__,
                            args=repr_args(*args, **kwargs),
                            url=self.driver.current_url,
                            console_errors=repr_console_errors(
                                console_errors
                            )
                        )
                    )
                    if is_check and hasattr(self.driver, "console_errors"):
                        self.driver.console_errors.append(
                            u"{console_errors}\n"
                            u"Url: {url}\n"
                            u"Action: {class_name}({by}={locator})"
                            u".{method}({args})\n"
                            u"{end_symbol}".format(
                                class_name=self.__class__.__name__,
                                by=self.by,
                                locator=self.locator,
                                method=fn.__name__,
                                args=repr_args(*args, **kwargs),
                                url=self.driver.current_url,
                                console_errors=repr_console_errors(
                                    console_errors, tabs=u""
                                ),
                                end_symbol=u"-" * 10
                            )
                        )
                except UnicodeDecodeError:
                    raise Exception(
                        u"Locator must be unicode, found cyrillic symbols."
                    )
            raise e
        return res
    return wrapper


def add_logger_to_base_element_classes(cls):
    for name, method in cls.__dict__.items():
        log.info(u"%s, %s" % (name, method))
        if (not name.startswith('_') and
                hasattr(method, '__call__') and name != "lookup"):
            setattr(cls, name, logger_for_element_methods(method))
    return cls


def logger_for_loading_page(fn):

    @wraps(fn)
    def wrapper(self, *args, **kwargs):
        res = fn(self, *args, **kwargs)
        if os.environ.get('CHECK_CONSOLE'):
            console_errors = find_console_browser_errors(self.driver)
            if console_errors:
                log.warning(
                    u"Browser console js error found:\n"
                    u"{tabs_format}{console_errors}\n"
                    u"{tabs_format}Url: {url}\n"
                    u"{tabs_format}Action: wait for page loaded ...".format(
                        tabs_format=TABS_FORMAT,
                        url=self.driver.current_url,
                        console_errors=repr_console_errors(console_errors)
                    )
                )
                if hasattr(self.driver, "console_errors"):
                    self.driver.console_errors.append(
                        u"{console_errors}\n"
                        u"Url: {url}\n"
                        u"Action: wait for page loaded ...\n"
                        u"{end_symbol}".format(
                            url=self.driver.current_url,
                            console_errors=repr_console_errors(
                                console_errors, tabs=u""
                            ),
                            end_symbol=u"-" * 10
                        )
                    )
        return res
    return wrapper


class LoggerFilter(logging.Filter):

    def filter(self, record):
        return record.levelno > 10


class Logger(object):

    def pytest_runtest_setup(self, item):
        item.capturelog_handler = LoggerHandler()
        item.capturelog_handler.setFormatter(logging.Formatter(
            u"%(asctime)-12s%(levelname)-8s%(message)s\n", u"%H:%M:%S"
        ))
        root_logger = logging.getLogger()
        item.capturelog_handler.addFilter(LoggerFilter())
        root_logger.addHandler(item.capturelog_handler)
        root_logger.setLevel(logging.NOTSET)

    @pytest.mark.hookwrapper
    def pytest_runtest_makereport(self, item, call):
        outcome = yield
        report = outcome.get_result()
        if hasattr(item, "capturelog_handler"):
            if call.when == 'teardown':
                root_logger = logging.getLogger()
                root_logger.removeHandler(item.capturelog_handler)
            if not report.passed:
                longrepr = getattr(report, 'longrepr', None)
                if hasattr(longrepr, 'addsection'):
                    captured_log = item.capturelog_handler.stream.getvalue()
                    if captured_log:
                        longrepr.sections.insert(
                            len(longrepr.sections),
                            (u'Captured log', u"\n%s" % captured_log, u"-")
                        )
            if call.when == 'teardown':
                item.capturelog_handler.close()
                del item.capturelog_handler


class LoggerHandler(logging.StreamHandler):

    def __init__(self):
        logging.StreamHandler.__init__(self)
        self.stream = py.io.TextIO()
        self.records = []

    def close(self):
        logging.StreamHandler.close(self)
        self.stream.close()


def request_logging(request, *args, **kwargs):
    log.info(
        u"HEADERS: {headers}\n"
        u"{tabs}METHOD: {method}\n"
        u"{tabs}LINK: {link}\n"
        u"{tabs}BODY: {body}\n"
        u"{tabs}RESPONSE CONTENT: "
        u"{ellipsis} ({length} symbols)\n"
        u"{tabs}RESPONSE HEADERS: {response_headers}\n"
        .format(
            headers=request.request.headers,
            method=request.request.method,
            link=request.url,
            body=request.request.body,
            content=re.sub(r'\s+', ' ', request.text)[:MAX_SYMBOLS],
            ellipsis=(
                u" ..." if len(request.text) > MAX_SYMBOLS else u""
            ),
            length=len(request.content),
            tabs=u" " * 12,
            response_headers=request.headers
        )
    )



# -*- coding: utf-8 -*-

import logging
import os
import traceback

import pytest
import requests

from selenium import webdriver
from selenium.common.exceptions import WebDriverException

from promium.assertions import WebDriverSoftAssertion, RequestSoftAssertion
from promium.core.common import get_screenshot
from promium.core.exceptions import PromiumException
from promium.core.support.settings import SettingsDrivers
from promium.device_config import CHROME_DESKTOP_1920_1080
from promium.logger import request_logging, logger_for_loading_page


log = logging.getLogger(__name__)

ENV_VAR = 'SE_DRIVER'
ENV_GRID = 'GRID_HOST'

DRIVERS = {
    'firefox': 'Firefox',
    'chrome': 'Chrome',
    'opera': 'Opera',
    'ie': 'IE',
    'edge': 'Edge'
}


def create_driver(device):
    """
    Examples SE_DRIVER:
        - 'chrome'
        - 'firefox'
        - 'opera'
        - 'ie'
        - 'edge'
    Examples GRID_HOST:
        - http://localhost:4444/wd/hub
        - htpp://se-grid.my:4444/wd/hub
        - htpp://selenoid:4444/wd/hub
    """
    driver = os.environ.get(ENV_VAR)
    grid_host = os.environ.get(ENV_GRID)
    if not driver:
        raise ValueError(
            u'Need choice Selenium WebDriver: {}'.format(DRIVERS.keys())
        )

    settings = SettingsDrivers.get(driver)
    settings.set_device(*device)
    if driver in DRIVERS and not grid_host:
        if driver == "chrome":
            return webdriver.Chrome(
                executable_path=settings.binary_path,
                chrome_options=settings.get_options(),
                desired_capabilities=settings.get_capabilities(),
            )
        elif driver == "firefox":
            return webdriver.Firefox(
                executable_path=settings.binary_path,
                firefox_profile=settings.get_preferences(),
                firefox_options=settings.get_options(),
                capabilities=settings.get_capabilities(),
            )
        elif driver == "opera":
            return webdriver.Opera(
                executable_path=settings.binary_path,
                options=settings.get_options(),
                desired_capabilities=settings.get_capabilities(),
            )
        elif driver == "ie":
            return webdriver.Ie(
                executable_path=settings.binary_path,
                options=settings.get_options(),
                capabilities=settings.get_capabilities(),
            )
        elif driver == "edge":
            return webdriver.Edge(
                executable_path=settings.binary_path,
                capabilities=settings.get_capabilities(),
            )
        return getattr(webdriver, DRIVERS[driver])()

    elif 'http' in grid_host:
        capabilities = settings.get_options().to_capabilities()
        if capabilities is None:
            raise ValueError(u'Unknown client specified: {}'.format(driver))

        capabilities.update(settings.get_capabilities())
        try:
            driver = webdriver.Remote(
                command_executor=grid_host,
                desired_capabilities=capabilities,
                browser_profile=settings.get_preferences()
            )
        except WebDriverException:
            log.warning(u"[SETUP] Second try for remote driver connection.")
            driver = webdriver.Remote(
                command_executor=grid_host,
                desired_capabilities=capabilities,
                browser_profile=settings.get_preferences()
            )
        return driver

    raise ValueError(
        u'Unknown driver specified: "{}", "{}"'.format(driver, grid_host)
    )


class TDPException(Exception):

    def __init__(self, *args):
        self.message = (
            "exception caught during execution test data preparing.\n"
            "Look at the original traceback:\n\n%s\n"
        ) % ("".join(traceback.format_exception(*args)))

    def __str__(self):
        return self.message


class TDPHandler:
    """
    TDP - Test Data Preparation
    context manager for handling any exception
    during execution test data preparing.
    We need to raise a specific custom exceptions.
    """

    def __init__(self):
        pass

    def __enter__(self):
        log.info("[TDP] Start test data preparing...")
        return self

    def __exit__(self, exc_type, exc_value, exc_tb):
        log.info("[TDP] Finish test data preparing")
        if exc_type:
            raise TDPException(exc_type, exc_value, exc_tb)
        return


class TestCase(object):
    test_case_url = None
    assertion_errors = None
    path_to_test = None

    def tdp_handler(self):
        """
        Use this context manager for prepare of test data only,
        not for business logic!
        """
        return TDPHandler()

    def get_path_to_test(self, method):
        return u"{path}.py -k {test_name}".format(
            path=u'/'.join(str(self.__module__).split(u'.')[1:]),
            test_name=method.__name__
        )

    def get_failed_test_command(self, name):
        if name == 'pytest':
            command = 'py.test'
        elif name == 'vagga':
            command = 'vagga run-tests --'
        else:
            raise PromiumException('Name must be "pytest" or "vagga"')
        return (
            u"{command} {path_to_test} --fail-debug-info --capturelog".format(
                command=command,
                path_to_test=self.path_to_test
            )
        )


class WebDriverTestCase(TestCase, WebDriverSoftAssertion):
    driver = None
    device = CHROME_DESKTOP_1920_1080  # default data
    excluded_browser_console_errors = []

    @logger_for_loading_page
    def get_url(self, url):
        self.driver.get(url)

    def check_console_errors(self):
        if hasattr(self.driver, "console_errors"):
            if self.driver.console_errors:
                browser_console_errors = self.driver.console_errors
                if self.excluded_browser_console_errors:
                    try:
                        return list(map(
                            lambda x: x, list(filter(
                                lambda x: x if not list(filter(
                                    lambda e: (
                                        True if e["msg"] in x and e["comment"]
                                        else False
                                    ),
                                    self.excluded_browser_console_errors
                                )) else None, browser_console_errors
                            ))
                        ))
                    except Exception as e:
                        raise PromiumException(
                            u"Please check your excluded errors list. "
                            u"Original exception is: %s" % e.message
                        )
                return browser_console_errors
        return []

    def setup_method(self, method):
        self.assertion_errors = []
        self.path_to_test = self.get_path_to_test(method)
        pytest.config.get_fail_debug = self.get_fail_debug
        pytest.config.assertion_errors = self.assertion_errors
        pytest.config.check_console_errors = self.check_console_errors
        if hasattr(method, 'device'):
            self.device = method.device.args[0]
        self.driver = create_driver(self.device)
        self.driver.console_errors = []
        if (
            'ie' not in os.environ['SE_DRIVER'] and
            'opera' not in os.environ['SE_DRIVER']
        ):
            self.driver.set_window_size(self.device.width, self.device.height)

    def teardown_method(self, method):
        self.driver.console_errors = []
        if self.driver:
            self.driver.quit()
        # TODO need fix
        # if not self.test_case_url and CHECK_TEST_CASE:
        #     raise PromiumException("Test don't have a test case url.")

    def get_fail_debug(self):
        """Failed test report generator"""
        alerts = 0
        try:
            while self.driver.switch_to.alert:
                alert = self.driver.switch_to.alert
                print(u'Unexpected ALERT: %s\n' % alert.text)
                alerts += 1
                alert.dismiss()
        except:
            if alerts != 0:
                print(u'')
            pass
        url = self.driver.current_url
        screenshot = get_screenshot(self.driver)
        pytest_failed_test_command = self.get_failed_test_command('pytest')
        vagga_failed_test_command = self.get_failed_test_command('vagga')
        return (
            'webdriver',
            url,
            screenshot,
            self.test_case_url,
            pytest_failed_test_command,
            vagga_failed_test_command
        )


class RequestTestCase(TestCase, RequestSoftAssertion):
    session = None

    def setup_method(self, method):
        self.path_to_test = self.get_path_to_test(method)
        self.session = requests.session()
        self.session.url = (
            u'Use self.get_response(url) for request tests or '
            u'util methods for api tests!'
        )
        self.assertion_errors = []
        pytest.config.assertion_errors = self.assertion_errors
        pytest.config.get_fail_debug = self.get_fail_debug

    def teardown_method(self, method):
        if self.session:
            self.session.close()
        # TODO need fix
        # if not self.test_case_url and CHECK_TEST_CASE:
        #     raise PromiumException("Test don't have a test case url.")

    def get_fail_debug(self):
        """Failed test report generator"""
        if not hasattr(self.session, 'status_code'):
            self.session.status_code = None
        pytest_failed_test_command = self.get_failed_test_command('pytest')
        vagga_failed_test_command = self.get_failed_test_command('vagga')
        return (
            u'request',
            self.session.url,
            self.session.status_code,
            self.test_case_url,
            pytest_failed_test_command,
            vagga_failed_test_command
        )

    def get_response(self, url, timeout=10, **kwargs):
        self.session.url = url
        self.session.status_code = None
        response = self.session.get(
            url,
            timeout=timeout,
            verify=False,
            hooks=dict(response=request_logging),
            **kwargs
        )
        self.session.status_code = response.status_code
        return response




import requests

from promium.core.base import Element
from promium.helpers import ParseUrl


class Link(Element):

    @property
    def href(self):
        return self.get_attribute("href")

    @property
    def parse_url(self):
        """
        Using parse_url to parse href. Can get attributes:
        - scheme
        - host
        - sub_domain
        - port
        - path
        - params
        - query
        - fragment
        - product_id
        """
        return ParseUrl(self.href)

    @property
    def response(self):
        return requests.get(self.href, verify=False, timeout=10)

    @property
    def get_status_code(self):
        """Gets status requests code from link"""
        return self.response.status_code



# -*- coding: utf-8 -*-

import pytest

from promium.test_case import WebDriverTestCase
from tests.urls import collect_url
from tests.pages.catalog_404_paga import Catalog404Page


MAIN_404_PAGE_TITLE = (
    u'К сожалению, такой страницы больше нет, или вы ввели неверный адрес.'
)
SUGGEST_404_PAGE_TITLE = (
    u'Воспользуйтесь поиском, или перейдите на главную страницу.'
)

SEARCH_TERM = 'search?search_term='


def get_expected_search_term_url(search_target):
    return collect_url('{search_term}{search_target}'.format(
        search_term=SEARCH_TERM,
        search_target=search_target
    ))


@pytest.mark.se
class Test404CatalogPage(WebDriverTestCase):

    def test_catalog_404_page_check_elements(self):
        catalog_404_page = Catalog404Page(self.driver)
        catalog_404_page.open()
        wrapper_block = catalog_404_page.wrapper_block
        self.soft_assert_equals(
            wrapper_block.logo_link.get_status_code,
            200,
            u'Не возможен переход на главную через лого.'
        )
        self.soft_assert_equals(
            wrapper_block.main_404_title.text,
            MAIN_404_PAGE_TITLE,
            u'Не правильно указан основной title на странице 404.'
        )
        self.soft_assert_equals(
            wrapper_block.suggest_404_title.text,
            SUGGEST_404_PAGE_TITLE,
            u'Не правильно указан suggest title на странице 404.'
        )
        self.soft_assert_equals(
            wrapper_block.suggest_404_title_link.get_status_code,
            200,
            u'Не возможен переход на главную через ссылку в suggest title.'
        )
        search_input = wrapper_block.search_block.search_input
        self.soft_assert_equals(
            search_input.placeholder,
            u'Я ищу...',
            u'Неправильный "placeholder" на странице 404.'

        )
        fake_search_query = 666
        search_input.send_keys(fake_search_query)
        wrapper_block.search_block.search_button.click()
        self.soft_assert_current_url(
            get_expected_search_term_url(fake_search_query)
        )




import time

import pytest
from mock import patch, MagicMock
from selenium.common.exceptions import TimeoutException

from promium.core.base import Element
from promium.core.exceptions import PromiumException
from promium.waits import (
    wait,
    wait_until,
    wait_until_not,
    wait_for_status_code
)
from promium.core.expected_conditions import is_present


FAKE_LOCATOR = ('', '')


@pytest.fixture()
def driver():
    return MagicMock(name='driver', **{
        'refresh.return_value': 'refresh OK',
        'session_id': '666'
    })


@pytest.mark.unit
def test_base_wait(driver):
    from selenium.webdriver.support.ui import WebDriverWait
    w = wait(driver)
    assert isinstance(w, WebDriverWait)
    assert hasattr(w, 'until')
    assert hasattr(w, 'until_not')


@pytest.mark.unit
@patch.object(Element, '_find_elements', return_value=True)
@pytest.mark.parametrize('seconds, msg', [
    (.01, None),
    (0, ''),
    (.2, 'test error message')
])
def test_wait_until(mock_element, seconds, msg, driver):
    element = Element(*FAKE_LOCATOR)
    start_time = time.time()
    wait_until(driver, is_present(element), seconds, msg)
    end_time = int(time.time() - start_time)
    mock_element.assert_called_once()
    assert end_time == 0, 'Not correct wait time duration'


@pytest.mark.unit
@patch.object(Element, '_find_elements', return_value=False)
@pytest.mark.parametrize('seconds, msg, ex_result', [
    (1, None, 1 * 2),
    (0, '', 1),
    (20, 'test error message', 20 * 2)
])
def test_wait_until_negative(mock_element, seconds, msg, ex_result, driver):
    element = Element(*FAKE_LOCATOR)
    start_time = time.time()
    with pytest.raises(TimeoutException, message='Element is found'):
        wait_until(driver, is_present(element), seconds, msg)
    end_time = int(time.time() - start_time)
    mock_element.assert_called()
    assert mock_element.call_count == ex_result
    assert end_time == seconds, 'Not correct wait time duration'


@pytest.mark.unit
@pytest.mark.parametrize('var, ex, seconds', [
    ([True], 1, 0),
    ([False, True], 2, 0),
    ([False, False, True], 3, 1),
    ([False] * 19 + [True], 20, 9),
])
def test_wait_until_count(var, ex, seconds, driver):
    with patch.object(Element, '_find_elements', side_effect=var) as el:
        element = Element(*FAKE_LOCATOR)
        start_time = time.time()
        wait_until(driver, is_present(element))
        end_time = int(time.time() - start_time)
        el.assert_called()
        assert el.call_count == ex
        assert end_time == seconds, 'Not correct wait time duration'


@pytest.mark.unit
@pytest.mark.parametrize('var, ex, seconds', [
    ([False], 1, 0),
    ([True, False], 2, 0),
    ([True, True, False], 3, 1),
    ([True] * 19 + [False], 20, 9),
])
def test_wait_until_not_positive(var, ex, seconds, driver):
    with patch.object(Element, '_find_elements', side_effect=var) as el:
        element = Element(*FAKE_LOCATOR)
        start_time = time.time()
        wait_until_not(driver, is_present(element))
        end_time = int(time.time() - start_time)
        el.assert_called()
        assert el.call_count == ex
        assert end_time == seconds, 'Not correct wait time duration'


@pytest.mark.unit
@patch.object(Element, '_find_elements', return_value=True)
@pytest.mark.parametrize('ex, seconds', [(1, 0), (2, 1), (10, 5)])
def test_wait_until_not_negative(mock_element, ex, seconds, driver):
    element = Element(*FAKE_LOCATOR)
    start_time = time.time()
    with pytest.raises(TimeoutException, message='Element is found'):
        wait_until_not(driver, is_present(element), seconds)
    end_time = int(time.time() - start_time)
    mock_element.assert_called()
    assert mock_element.call_count == ex
    assert end_time == seconds, 'Not correct wait time duration'


@pytest.mark.unit
@pytest.mark.parametrize('var, ex, ex_sec', [
    ([True], 1, 0),
    ([False, True], 2, 0),
    ([False, False, True], 3, 1),
    ([False, False] * 9 + [True], 19, 9),
])
def test_wait_until_with_reload(var, ex, ex_sec, driver):
    with patch.object(Element, '_find_elements', side_effect=var) as el:
        element = Element(*FAKE_LOCATOR)
        start_time = time.time()
        wait_until(
            driver=driver,
            expression=is_present(element, with_refresh=True),
            seconds=10
        )
        end_time = int(time.time() - start_time)
        el.assert_called()
        assert el.call_count == ex
        assert end_time == ex_sec, 'Not correct wait time duration'
        driver.refresh.assert_called()
        assert driver.refresh.call_count == ex


@pytest.mark.unit
@patch.object(Element, '_find_elements', return_value=False)
@pytest.mark.parametrize('ex, sec', [(1, 0), (2, 1), (20, 10)])
def test_wait_until_with_reload_negative(mock_element, ex, sec, driver):
    element = Element(*FAKE_LOCATOR)
    start_time = time.time()
    with pytest.raises(TimeoutException):
        wait_until(
            driver=driver,
            expression=is_present(element, with_refresh=True),
            seconds=sec
        )
    end_time = int(time.time() - start_time)
    mock_element.assert_called()
    assert end_time == sec, 'Not correct wait time duration'
    driver.refresh.assert_called()
    assert driver.refresh.call_count == ex


@pytest.mark.skip('Not implemented')
@pytest.mark.unit
def test_wait_for_alert():
    pass


@pytest.mark.skip('Not implemented')
@pytest.mark.unit
def test_wait_for_alert_negative():
    pass


@pytest.mark.unit
@pytest.mark.parametrize('tries', [1, 2, 10, 15])
def test_wait_for_status_code(tries):
    start_time = time.time()
    wait_for_status_code('http://google.com/page?id=666', 404, tries)
    end_time = int(time.time() - start_time)
    assert end_time == 0


@pytest.mark.unit
@pytest.mark.parametrize('tries', [1, 2, 5])
def test_wait_for_status_code_negative(tries):
    start_time = time.time()
    with pytest.raises(PromiumException):
        wait_for_status_code('http://google.com/page?id=666', 200, tries)
    end_time = int(time.time() - start_time)
    assert end_time == tries


@pytest.mark.skip('Not implemented')
@pytest.mark.unit
def test_wait_until_new_window_is_opened():
    pass


@pytest.mark.skip('Not implemented')
@pytest.mark.unit
def test_wait_until_new_window_is_opened_negative():
    pass



# -*- coding: utf-8 -*-

from collections import namedtuple


USER_AGENT_ANDROID = (
    u"Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) "
    u"AppleWebKit/537.36 (KHTML, like Gecko) "
    u"Chrome/62.0.3202.62 Mobile Safari/537.36"
)

USER_AGENT_IOS = (
    u"Mozilla/5.0 (iPhone; CPU iPhone OS 9_1 like Mac OS X) "
    u"AppleWebKit/601.1.46 (KHTML, like Gecko) "
    u"Version/9.0 Mobile/13B143 Safari/601.1"
)

Device = namedtuple('Device', 'width height user_agent')

CHROME_DESKTOP_1920_1080 = Device(1920, 1080, '')
CHROME_DESKTOP_1024_768 = Device(1024, 768, '')

ANDROID_360_640 = Device(360, 640, USER_AGENT_ANDROID)
ANDROID_1280_800 = Device(1280, 800, USER_AGENT_ANDROID)
ANDROID_1080_1920 = Device(1080, 1920, USER_AGENT_ANDROID)
IOS_750_1334 = Device(750, 1334, USER_AGENT_IOS)
IOS_1080_1920 = Device(1080, 1920, USER_AGENT_IOS)




from promium.core.elements.input_field import InputField
from promium.waits import wait_for_ajax


class UploadButton(InputField):

    def upload_file(self, file_path):
        """Uploads file by file path"""
        self.send_keys(file_path)
        wait_for_ajax(self.driver)



# -*- coding: utf-8 -*-

import json
import re

from json_checker import Checker, CheckerError
from six import text_type, string_types

from promium.core.common import get_screenshot


class BaseSoftAssertion(object):

    # TODO is not cleaned in unit tests need use __init__
    assertion_errors = []

    # TODO need fix default message
    @staticmethod
    def _format_message(msg):
        return u"{}\n".format(msg) if msg else u""

    @staticmethod
    def convert_container(container):
        return json.dumps(
            container, indent=4, sort_keys=True, ensure_ascii=False,
            default=lambda obj: text_type(obj)
        ) if not isinstance(container, string_types) else container

    @staticmethod
    def get_text_with_ignore_whitespace_symbols(text):
        """Return text excluding spaces and whitespace_symbols"""
        text_without_whitespace_symbols = (
            text
            .replace(u'\t', u' ')
            .replace(u'\v', u' ')
            .replace(u'\r', u' ')
            .replace(u'\n', u' ')
            .replace(u'\f', u' ')
            .strip()
        )
        text_list = text_without_whitespace_symbols.split(u' ')
        text_list_without_space = [word for word in text_list if word]
        # TODO what happen?
        needful_text = u' '.join(text_list_without_space)
        return needful_text

    def soft_assert_true(self, expr, msg=None):
        """Check that the expression is true."""
        if not expr:
            error = u"Is not true." if not msg else msg
            self.assertion_errors.append(error)
            return error

    def soft_assert_false(self, expr, msg=None):
        """Check that the expression is false."""
        if expr:
            message = u"Is not false." if not msg else msg
            self.assertion_errors.append(message)
            return message

    def soft_assert_equals(self, current, expected, msg=None):
        """Just like self.soft_assert_true(current == expected)"""
        message = (
            u"{base_msg}"
            u"Current - {current}\n"
            u"Expected - {expected}\n".format(
                base_msg=self._format_message(msg),
                current=text_type(current),
                expected=text_type(expected)
            )
        )
        if type(current) != type(expected):
            message += (
                    u"Current and expected has different data types: "
                    u"current is %s, expected is %s" % (
                        type(current), type(expected)
                    )
            )
        return self.soft_assert_true(current == expected, message)

    def soft_assert_not_equals(self, current, expected, msg=None):
        """Just like self.soft_assert_true(current != expected)"""
        message = (
            u"{base_msg}"
            u"Current - {current}\n"
            u"Expected - {expected}\n".format(
                base_msg=self._format_message(msg),
                current=text_type(current),
                expected=text_type(expected)
            )
        )
        self.soft_assert_false(current == expected, message)

    def soft_assert_in(self, member, container, msg=None):
        """Just like self.soft_assert_true(member IN container)"""
        error = (
            u"{base_msg}"
            u"{member} not found in {container}\n".format(
                base_msg=self._format_message(msg),
                member=member,
                container=self.convert_container(container)
            )
        )
        return self.soft_assert_true(member in container, error)

    def soft_assert_not_in(self, member, container, msg=None):
        """Just like self.soft_assert_true(member NOT IN container)"""
        error = (
            u"{base_msg}"
            u"{member} unexpectedly found in {container}\n".format(
                base_msg=self._format_message(msg),
                member=member,
                container=self.convert_container(container)
            )
        )
        return self.soft_assert_true(member not in container, error)

    def soft_assert_less_equal(self, a, b, msg=None):
        """Just like self.soft_assert_true(a <= b)"""
        error = u"{base_msg}{a} not less than or equal to {b}\n".format(
            base_msg=self._format_message(msg), a=a, b=b
        )
        return self.soft_assert_true(a <= b, error)

    def soft_assert_less(self, a, b, msg=None):
        """Just like self.soft_assert_true(a < b)"""
        error = u"{base_msg}{a} not less than {b}\n".format(
            base_msg=self._format_message(msg), a=a, b=b
        )
        return self.soft_assert_true(a < b, error)

    def soft_assert_greater_equal(self, a, b, msg=None):
        """Just like self.soft_assert_true(a >= b)"""
        error = u"{base_msg}{a} not greater than or equal to {b}\n".format(
            base_msg=self._format_message(msg), a=a, b=b
        )
        return self.soft_assert_true(a >= b, error)

    def soft_assert_greater(self, a, b, msg=None):
        """Just like self.soft_assert_true(a > b)"""
        error = u"{base_msg}{a} not greater than {b}\n".format(
            base_msg=self._format_message(msg), a=a, b=b
        )
        return self.soft_assert_true(a > b, error)

    def soft_assert_regexp_matches(self, text, expected_regexp, msg=None):
        """Fail the test unless the text matches the regular expression."""
        pattern = re.compile(expected_regexp)
        result = pattern.search(text)

        if not result:
            error = (
                u"{base_msg}"
                u"Regexp didn't match."
                u"Pattern {pattern} not found in {text}\n".format(
                    base_msg=self._format_message(msg),
                    pattern=text_type(pattern.pattern),
                    text=text_type(text)
                )
            )
            self.assertion_errors.append(error)
            return error

    def soft_assert_disable(self, element, msg=None):
        """Check that the obj hasn't attribute."""
        error = u"{base_msg}{default_msg}".format(
            base_msg=self._format_message(msg),
            default_msg=u"%s is not Disable.\n" % element if not msg else u""
        )
        return self.soft_assert_true(element.get_attribute("disabled"), error)

    def soft_assert_is_none(self, obj, msg=None):
        """Same as self.soft_assert_true(obj is None)."""
        error = u"{base_msg}{default_msg}".format(
            base_msg=self._format_message(msg),
            default_msg=u"%s is not None.\n" % obj if not msg else u""
        )
        return self.soft_assert_true(obj is None, error)

    def soft_assert_is_not_none(self, obj, msg=None):
        """Included for symmetry with self.soft_assert_is_none."""
        error = u"{base_msg}{default_msg}".format(
            base_msg=self._format_message(msg),
            default_msg=u"Unexpectedly None.\n" if not msg else u""
        )
        return self.soft_assert_true(obj is not None, error)

    def soft_assert_is_instance(self, obj, cls, msg=None):
        """Same as self.soft_assert_true(isinstance(obj, cls))"""
        error = u"{base_msg}{default_msg}".format(
            base_msg=self._format_message(msg),
            default_msg=u"%s is not an instance of %s.\n" % (obj, cls) if
            not msg else u""
        )
        return self.soft_assert_true(isinstance(obj, cls), error)

    def soft_assert_equals_text_with_ignore_spaces_and_register(
        self,
        current_text,
        expected_text,
        msg=u'Invalid checked text.'
    ):
        """Checking of text excluding spaces and register"""
        current = self.get_text_with_ignore_whitespace_symbols(current_text)
        expected = self.get_text_with_ignore_whitespace_symbols(expected_text)
        error_pattern = (
            u"%s\nCurrent text without formating: %s"
            u"\nExpected text without formating: %s"
        )
        if not current:
            msg = u"Warning: current text is None!"
        self.soft_assert_equals(
            current.lower(),
            expected.lower(),
            error_pattern % (msg, current_text, expected_text)
        )

    def soft_assert_schemas(self, current, expected, msg=''):
        """
        Example:
            {'test1': int} == {'test1: 1}

        :param dict current: current response
        :param dict expected: expected dict(key: type)
        :param str msg:
        :return error
        """
        try:
            Checker(expected, soft=True).validate(current)
        except CheckerError as e:
            error = u'{}\n{}'.format(msg, e)
            self.assertion_errors.append(error)
            return error

    def assert_keys_and_instances(
        self,
        actual_dict,
        expected_dict,
        can_be_null=None,
        msg=None
    ):
        """
        :param dict actual_dict:
        :param dict expected_dict:
        :param list | None can_be_null: must be if default value None
        :param basestring msg:
        """
        assert actual_dict, u'Actual dict is empty, check your data'

        self.soft_assert_equals(
            sorted(actual_dict.keys()),
            sorted(expected_dict.keys()),
            u'Wrong keys list.'
        )
        for actual_key, actual_value in actual_dict.items():
            self.soft_assert_in(
                member=actual_key,
                container=expected_dict,
                msg=u'Not expected key "{member}".'.format(member=actual_key)
            )
            if actual_key in expected_dict:
                expected_value = (
                    type(None) if
                    actual_value is None and
                    actual_key in (can_be_null or []) else
                    expected_dict[actual_key]
                )
                self.soft_assert_true(
                    expr=isinstance(actual_value, expected_value),
                    msg=(
                        u'Wrong object instance class.\n'
                        u'Key "{key}" value is "{current_type}", '
                        u'expected "{expected_type}". {msg}'.format(
                            key=actual_key,
                            current_type=type(actual_value),
                            expected_type=expected_value,
                            msg=u'(%s)' % msg if msg else u''
                        )
                    )
                )


class RequestSoftAssertion(BaseSoftAssertion):

    @property
    def url(self):
        return self.session.url

    def base_msg(self, msg=None):
        return u"{url}{exception}".format(
            url=u"\n%s" % self.url if self.url else u"",
            exception=(u"\n%s" % msg) if msg else u""
        )


class WebDriverSoftAssertion(BaseSoftAssertion):

    @property
    def driver(self):
        return self.driver

    @property
    def url(self):
        return self.driver.current_url

    @property
    def assertion_screenshot(self):
        return get_screenshot(self.driver)

    def base_msg(self, msg=None):
        return u"\n{url}\nScreenshot - {screenshot}{exception}".format(
            url=self.url,
            screenshot=self.assertion_screenshot,
            exception=(u"\n%s" % msg) if msg else u""
        )

    def soft_assert_page_title(self, expected_title, msg=u"Wrong page title."):
        self.soft_assert_equals(self.driver.title, expected_title, msg)

    def soft_assert_current_url(self, expected_url, msg=u"Wrong current url."):
        self.soft_assert_equals(self.url, expected_url, msg)

    def soft_assert_current_url_contains(self, url_mask, msg=None):
        self.soft_assert_in(url_mask, self.url, msg)

    def soft_assert_element_is_present(self, element, msg=None):
        if not msg:
            msg = (
                u"Element %s=%s is not present on page at current time.\n" % (
                    element.by, element.locator
                )
            )
        self.soft_assert_true(element.is_present(), msg)

    def soft_assert_element_is_not_present(self, element, msg=None):
        if not msg:
            msg = u"Element %s=%s is found on page.\n" % (
                element.by, element.locator
            )
        self.soft_assert_false(element.is_present(), msg)

    def soft_assert_element_is_displayed(self, element, msg=None):
        if not msg:
            msg = u"Element %s=%s is not visible to a user.\n" % (
                element.by, element.locator
            )
        self.soft_assert_true(element.is_displayed(), msg)

    def soft_assert_element_is_not_displayed(self, element, msg=None):
        if not msg:
            msg = u"Element %s=%s is visible to a user.\n" % (
                element.by, element.locator
            )
        self.soft_assert_false(element.is_displayed(), msg)
